#!/bin/bash

#####################################################################
#     Password capture, hash and import tool for PasswordServer     #
#                                                                   #
#   This script is utilized in order to capture password changes    #
# from PasswordServer, generate an SHA1 hash of the password, then  #
#  apply that password into OpenLDAP onto the user record.          #
#  The LDAP attribute pagerdefined by the inetOrgPerson object-     #
#  class, will store the hash of the password.                      #
#                                                                   #
#   Required values:                                                #
#       Specification of domain in echo command                     #
#       User with admin rights to directory in ldapmodify command   #
#                                                                   #
#   Additional steps:                                               #
#       Google Apps Directory Sync will need to run to apply        #
#       password to Google User Database.                           #
#                                                                   #
#   Installation:                                                   #
#       Copy script to /usr/sbin/authserver/tools                   #
#       chown script to root and set permissions to 700             #
#       Add script name (path not needed) to ExternalCommand key    #
#       /Library/Preferences/com.apple.passwordserver.plist         #
#       Unload / reload passwordserver via launchd (or restart)     #
#                                                                   #
#   Notes:                                                          #
#       Prior to running command, ensure you have a tested archive  #
#       of your diretory database for safety.                       #
#       While testing, keep one web-browser session connected to    #
#       your Google Apsp domain.  This way if you run into any      #
#       problems you have one active session to reset a password    #
#       if an invalid password gets sent to Google on your admin    #
#       account.                                                    #
#       You may wish to create an account and modify LDAP ACLS      #
#       to permit only a few accounts access to read pager          #
#       attriibute since hash values of easy password can be found. #
#####################################################################

# Get password from stdin.  This will be the password change running through password server.
read password

# Hash the password using the SHA1 method and store into password.

password=`echo -n $password | /usr/bin/openssl dgst -sha1 -hex`

# Create LDIF file for LDAP modification.  This will import the value into LDAP attribute pager.
# If you wish to change the attribute, change both instances of pager.  Also, make sure to keep the
# \n inplace as currently formatted.
#
# REQUIRED CHANGES: specify your domain below, replacing dc=YOUR,dc=DOMAIN

LDIFMOD="/usr/sbin/authserver/tools/modify.ldif"
touch $LDIFMOD
echo -e "dn: uid=$1,cn=users,dc=YOUR,dc=DOMAIN\nchangetype: modify\nreplace: pager\npager: $password" > $LDIFMOD

# ldapmodify command to import previously generated LDIF file back into directory.
# You will need to specify an account with admin rights to your LDAP database here.
#
# REQUIRED CHANGES: admin username for uid=SOMEADMIN.
#                   Domain information replaing dc=YOUR,dc=DOMAIN
#                   password for userwith write access for PASSWORD

ldapmodify -xD uid=SOMEADMIN,cn=users,dc=YOUR,dc=DOMAIN -w PASSWORD -f $LDIFMOD -v

# Remove LDIF file for security purposes.

srm $LDIFMOD

exit 0